import { useState, useEffect } from "react";
import {
  Box,
  Typography,
  Button,
  TextField
} from "@mui/material";
import AddIcon from '@mui/icons-material/Add';
import RemoveIcon from '@mui/icons-material/Remove';
import { useSelector, useDispatch } from "react-redux";
import { createCart } from "../../redux/slices/cartSlice";

// Utility function to check if an object is empty
const isEmptyObject = (obj) => Object.keys(obj).length === 0 && obj.constructor === Object;

// Check if every key in the target object matches the source object
const hasMatchingProperties = (source, target) => {
  return Object.entries(target).every(([key, value]) => source[key] === value);
};

const ProductDetails = () => {

  const dispatch = useDispatch()
  const product = useSelector((state) => state.products.product.data)
  const cart = useSelector((state) => state.cart.cart)

  const [quantity, setQuantity] = useState(1)
  const [variantSelection, setVariantSelection] = useState({
    price: 0,
    currencyCode: "",
    quantityAvailable: 0
  })
  const [variants, setVariants] = useState([])

  useEffect(() => {
    if (product) {

      console.log("Product: ", product)

      let productVariants = product?.variants?.nodes

      // get all variant type
      if (!productVariants || productVariants?.length == 0) return

      // we want to get the title for each variant
      const uniqueOptions = {};
      productVariants.forEach(variant => {
        variant.selectedOptions.forEach(option => {
          if (!uniqueOptions[option.name]) {
            uniqueOptions[option.name] = new Set();
          }
          uniqueOptions[option.name].add(option.value);
        });
      });

      const VariantsArr = Object.entries(uniqueOptions).map(([key, valueSet]) => ({
        name: key,
        options: Array.from(valueSet),
      }));

      // get variants value
      setVariants(VariantsArr)

      // setting Initial value for variants selection
      setVariantSelection((prev) => {

        let newVariantSelection = { ...prev }

        // setting inital selection
        VariantsArr.forEach(({ name, options }) => {
          newVariantSelection = { ...newVariantSelection, [name]: options[0] }
        })

        // find variant price if it all match initial variant selection
        for (const { selectedOptions, price, id, quantityAvailable } of productVariants) {
          let { amount, currencyCode } = price;
        
          // Convert array to object
          const optionsObject = selectedOptions.reduce(
            (a, { name, value }) => ({ ...a, [name]: value }),
            {}
          );
        
          if (hasMatchingProperties(newVariantSelection, optionsObject)) {
            newVariantSelection = { ...newVariantSelection, amount, currencyCode, id, quantityAvailable };
            break; // Exit the loop when condition is met
          }
        }

        return newVariantSelection

      })

    }

  }, [product])

  useEffect(() => {
    console.log("variantSelection: ", variantSelection)
  }, [variantSelection])

  const handleVariantClick = (name, value) => {
    setVariantSelection({ ...variantSelection, [name]: value })

    setVariantSelection((prev) => {

      let newVariantSelection = { ...prev }
      newVariantSelection = { ...newVariantSelection, [name]: value }
      let productVariants = product?.variants?.nodes

      // find variant price if it all match initial variant selection
      for (const { selectedOptions, price, id, quantityAvailable } of productVariants) {
        let { amount, currencyCode } = price;
      
        // Convert array to object
        const optionsObject = selectedOptions.reduce(
          (a, { name, value }) => ({ ...a, [name]: value }),
          {}
        );
      
        if (hasMatchingProperties(newVariantSelection, optionsObject)) {
          newVariantSelection = { ...newVariantSelection, amount, currencyCode, id, quantityAvailable };
          if(quantityAvailable == 0) setQuantity(0)
          else setQuantity(1)
          break; // Exit the loop when condition is met
        }
      }


      return newVariantSelection

    })

  }

  const handleCart = () => {

    let cartHistory = localStorage.getItem('amber-cart');
    cartHistory = cartHistory ? JSON.parse(cartHistory) : {};

    // if we got no cart, then create a new one
    if (isEmptyObject(cart) || isEmptyObject(cartHistory)) {

      dispatch(createCart());

    } else {

      // // Update cart content
      // const existingItemIndex = cartHistory.lines?.findIndex(
      //   (line) => line.merchandiseId === item.merchandiseId
      // );

      // if (existingItemIndex !== -1) {
      //   // If the item exists, update the quantity
      //   cartHistory.lines[existingItemIndex].quantity += item.quantity;
      // } else {
      //   // If the item doesn't exist, add it
      //   cartHistory.lines = [...(cartHistory.lines || []), item];
      // }

      // // Update Redux store and localStorage
      // dispatch(updateCart(cartHistory)); // Redux action to update cart
      // localStorage.setItem('amber-cart', JSON.stringify(cartHistory));
    }

  }

  const handleIncrement = () => {
    setQuantity((prevQuantity) => prevQuantity + 1);
  };

  const handleDecrement = () => {
    setQuantity((prevQuantity) => (prevQuantity > 1 ? prevQuantity - 1 : 1));
  };


  return (
    <Box sx={{ position: "relative" }}>
      {/* Flex Container */}
      <Box sx={{ display: "flex", flexDirection: "column", gap: 2 }}>
        {/* Section 1: Product Info */}
        <Box>
          <Typography variant="h5" sx={{ fontWeight: "bold", mb: 2 }}>
            {product.title}
          </Typography>
          <Typography variant="body2" color="text.secondary">
            {product?.collections?.nodes[0]?.title}
          </Typography>
          <Typography
            variant="body1"
            sx={{ fontWeight: "bold" }}
          >
            {`${variantSelection.currencyCode} ${parseFloat(variantSelection.amount).toFixed(2)}`}
          </Typography>
          <Typography
            variant="body1"
          >
            IN STOCK: {`${variantSelection?.quantityAvailable}`}
          </Typography>
        </Box>

        {/* Section 2: Variants */}
        <Box>

          {variants.map(({ name, options }, index) => {

            return (
              <>
                <Typography variant="body1" sx={{ fontWeight: "bold", color: "#000", mb:1 }}>
                  {name}
                </Typography>

                <Box display="flex" gap={2} sx={{ mb: 2 }}>
                  {options?.map((value) => (
                    <Button
                      key={value}
                      variant={variantSelection[name] === value ? "contained" : "outlined"}
                      color={variantSelection[name] === value ? "primary" : "primary"}
                      sx={{ color: variantSelection[name] === value ? "#FFF" : "#000" }}
                      onClick={() => handleVariantClick(name, value)}
                    >
                      {value}
                    </Button>
                  ))}
                </Box>
              </>
            )

          })}

        </Box>


        {/* Section 3: Quantity */}
        <Box sx={{ mb: 5 }}>
          <Typography variant="body1" sx={{ fontWeight: "bold", color: "#000", mb: 2 }}>
            Quantity
          </Typography>
          <Box display="flex" alignItems="center" gap={2}>
            <Button
              variant="contained"
              color="primary"
              sx={{ width: "35px" }}
              disabled={variantSelection?.quantityAvailable == 0}
              onClick={handleDecrement}
            >
              <RemoveIcon />
            </Button>

            <TextField
              value={quantity}
              inputProps={{ readOnly: true, style: { textAlign: 'center' } }}
              size="small"
              sx={{ width: "100px" }}
              variant="outlined"
            />

            <Button
              variant="contained"
              color="primary"
              sx={{ width: "35px" }}
              disabled={variantSelection?.quantityAvailable == 0}
              onClick={handleIncrement}
            >
              <AddIcon />
            </Button>
          </Box>
        </Box>

        {/* Section 4: Description */}
        <Box sx={{ mb: 5 }}>
          <Typography variant="body1" sx={{ fontWeight: "bold", color: "#000" }}>
            Description
          </Typography>
          <Typography variant="body1" color="text.secondary" sx={{ fontWeight: "400" }}>
            <div dangerouslySetInnerHTML={{ __html: product.descriptionHtml }}></div>
          </Typography>
        </Box>

        <Button
          onClick={() => { handleCart() }}
          variant="contained"
          color="common.black"
          fullWidth
          sx={{
            backgroundColor: (theme) => theme.palette.common.black,
            color: "white",
            textTransform: "none",
            mt: 2,
            "&:hover": {
              backgroundColor: (theme) => theme.palette.grey[900],
            },
          }}
        >
          ADD TO CART
        </Button>
      </Box>

    </Box>
  );
};

export default ProductDetails;
